/*~******************************************************************************************
*	~Project:	"Alone"																		*
*	~Author:	Tim Beaudet																	*
*	~Modified:	12/16/2011																	*
*	~Purpose:	Football field for some basic gameplay.										*
******************************************************************************************~*/

#include "Alone.h"
#include "CFootball.h"
#include "CFootballField.h"
#include "CFootballPlayer.h"

static const float BallSizeNormal = 0.4f;
static const float BallSizeInAir = 0.6f;

CFootball::CFootball() : mFootballField(nullptr), mPlayer(nullptr), mThrower(nullptr),
	mIsInAir(false)
{
	SetTexture(Textures::Football);
	mSprite.SetTextureCoords(0.5f, 0.0f, 0.5f, 0.5f);
	mWidth = mHeight = BallSizeNormal;
}

CFootball::~CFootball(void)
{

}

void CFootball::Update(float dt)
{
	if (nullptr != mPlayer)
	{
		Vector3 fwd = mPlayer->GetForward();
		SetForward(Vector3(-fwd.z, 0.0f, fwd.x));
	}

	if (false == mIsInAir)
		return;

	Vector3 pos = GetPosition();

	float dist;
	Vector3 toTarget;
	Vector3Subtract(&toTarget, &mTargetPos, &pos);
	Vector3NormalizeMag(&toTarget, &toTarget, dist);
	SetForward(toTarget);

	float moveAmt = FootballSpeed * dt;
	if (dist < moveAmt)
	{
		moveAmt = dist;
		mIsInAir = false;
		mWidth = mHeight = BallSizeNormal;
	}

	Vector3Scale(&toTarget, &toTarget, moveAmt);
	Vector3Add(&pos, &pos, &toTarget);
	SetPosition(pos);

	if (false == mIsInAir)
	{	//This just happened, did someone catch the ball?
		CFootballField::PlayerItr itr = mFootballField->BeginPlayer();
		CFootballField::PlayerItr itrEnd = mFootballField->EndPlayer();
		CFootballPlayer *ply = nullptr;
		for ( ; itr != itrEnd; ++itr)
		{			
			if (mThrower == (ply = *itr) || false == ply->CanCatch())
				continue;

			Vector3 plyPos = ply->GetPosition();
			Vector3 diff;
			Vector3Subtract(&diff, &plyPos, &GetPosition());
			float distSq = Vector3MagnitudeSq(&diff);
			float catchDist = ply->GetRadius() + 0.75f;

			if (distSq < catchDist * catchDist)
			{
				if ("player" == ply->GetRoleName())
					Sounds::PlaySfx("oncatch");

				ply->SetFootball(this);
				SetPlayer(ply);
				break;
			}
		}
	}
}

void CFootball::ThrowTo(const Vector3 &pos)
{
	mTargetPos = pos;
	mIsInAir = true;
	mThrower = mPlayer;
	mPlayer->SetFootball(nullptr);
	mPlayer = nullptr;

	mWidth = mHeight = BallSizeInAir;
}

void CFootball::SetPlayer(CFootballPlayer *plyrWithBall)
{
	if (mPlayer == plyrWithBall)
		return;

	if (nullptr != mPlayer)
		mPlayer->SetFootball(nullptr);

	mPlayer = plyrWithBall;	
	if (nullptr != mPlayer)
		mPlayer->SetFootball(this);
}
