/*~******************************************************************************************
*	~Project:	"Alone"																		*
*	~Author:	Tim Beaudet																	*
*	~Modified:	12/16/2011																	*
*	~Purpose:	Football field for some basic gameplay.										*
******************************************************************************************~*/

#ifndef _Alone_CFootballField_h_
#define _Alone_CFootballField_h_

class CPopupInfo;
class CDustBall;
class CFootball;
class CFootballPlayer;
enum PlayerType;

enum DownReasons
{
	DOWN_CONTACT,
	DOWN_INTERCEPT,
	DOWN_OUTBOUNDS,
	DOWN_INCOMPLETE,
	DOWN_TOUCHDOWN,
	DOWN_SAFETY,
	DOWN_MAX,
};

class CFootballField
{
public:
	typedef std::vector<CFootballPlayer*> PlayerContainer;
	typedef PlayerContainer::iterator PlayerItr;
	typedef PlayerContainer::const_iterator PlayerCItr;
private:
	char *mTiles;
	unsigned char mCols;
	unsigned char mRows;
	
	CDustBall *mDustBall;
	CPopupInfo *mPopupInfo;

	unsigned char mRowOfYard1;
	unsigned char mColOfLeftEdge;
	unsigned char mColOfRightEdge;

	float mGameTimeLeft;
	unsigned char mAwayScore;
	unsigned char mHomeScore;

	bool mHasGameConcluded;
	char mCurrentDown;
	char mNextFirstDown;
	char mLineOfScrimmage;
	char mBallPlacement;

	int mTotalYardage;
	int mTotalDownsPlayed;
	int mIncompletePasses;
	int mTotalInterceptions;
	int mTotalPasses;

	bool mIsInPlay;
	bool mReadyToPlay;
	bool mIsScoring;

	PlayerContainer mPlayers;

	Vector3 mPlayerStart;
	CFootball *mFootball;
public:

	CFootballField(void);
	~CFootballField(void);
	
	void Update(float dt);
	void Draw(void) const;

	void Reset4thQuarter(float timeLeft, unsigned char awayScore, char lineOfScrim);
	void ResetForDown();
	void ResetAfterPunt();
	
	void ClearFieldAfterPlay(void);
	CFootballPlayer* PlacePlayer(PlayerType type, FootballTeam team, const Vector3 &fieldPos);

	void HikeFootball(void);

	const Vector3& GetPlayerPosition() const;
	const Vector3& GetFootballPosition() const;
	Vector3 GetCenterAt(char yrd = -1) const;
	float LineOfScrimmageZ(void) const;
	float FirstDownZ(void) const;

	char FieldPosition(const Vector3 &pos) const;
	char FieldPosition(float posZ) const;

	void SpawnDust(const Vector3 &at);
	bool IsReadyToPlay(void) const { return mReadyToPlay; }
	bool IsInPlay(void) const { return mIsInPlay; }

	PlayerCItr BeginPlayer(void) const { return mPlayers.begin(); }
	PlayerCItr EndPlayer(void) const { return mPlayers.end(); }
	PlayerItr BeginPlayer(void) { return mPlayers.begin(); }
	PlayerItr EndPlayer(void) { return mPlayers.end(); }

	const CFootballPlayer* FindPlayerWithRole(const std::string &roleName);
	const CFootballPlayer* GetPlayerWithFootball(void) const;

private:
	CFootballField(const CFootballField &obj);
	CFootballField& operator=(const CFootballField &obj);

	void DrawMarkerLine(char yrd, unsigned char tileID) const;
	void DrawTile(unsigned char tileID, const Vector3 &at) const;
	void DrawTile(float tx, float ty, const Vector3 &at) const;

	void ComputeFirstDown(void);

	void EndOfDown(DownReasons reason);
	void OnTouchDown(void);
	void OnSafety(void);
	void OnGameEnd(void);
};

#endif _Alone_CFootballField_h_