/*~******************************************************************************************
*	~Project:	"Alone"																		*
*	~Author:	Tim Beaudet																	*
*	~Modified:	12/16/2011																	*
*	~Purpose:	Football players, AI and User.												*
******************************************************************************************~*/

#include "Alone.h"
#include "CFootballPlayer.h"
#include "CFootballField.h"
#include "CFootball.h"

CFootballPlayer::CFootballPlayer(void) : mFootballField(nullptr), mRunModifier(1.0f), mRadius(0.4f), mFootball(nullptr)
{
	mSprite.mWidth = mSprite.mHeight = 0.66f;
	mSprite.SetTexture(Textures::HomePlayer);
	mSprite.SetPosition(Vector3(5.0f, 0.0f, 5.0f));
}

CFootballPlayer::~CFootballPlayer(void)
{
	if (nullptr != mFootball)
		mFootball->SetPlayer(nullptr);
}

void CFootballPlayer::Update(float dt)
{
	if (nullptr != mFootball)
	{
		Vector3 tmp;
		Vector3 pos = GetPosition();
		Vector3Scale(&tmp, &mSprite.GetForward(), 0.25f);
		Vector3Add(&pos, &pos, &tmp);
		mFootball->SetPosition(pos);
	}
}

void CFootballPlayer::Draw(void) const
{
	mSprite.Draw();
}

void CFootballPlayer::RunForward(float amt)
{
	Vector3 pos = mSprite.GetPosition();
	Vector3 fwd = mSprite.GetForward();
	Vector3 move;
	Vector3Scale(&move, &fwd, amt);
	Vector3Add(&pos, &pos, &move);

	bool isColliding = false;
	Vector3 dist;
	for (CFootballField::PlayerCItr itr = mFootballField->BeginPlayer(), itrEnd = mFootballField->EndPlayer(); itr != itrEnd; ++itr)
	{
		CFootballPlayer *other = *itr;
		if (this == other)
			continue;

		//float collideDist = mRadius + other->mRadius;
		//if (GetTeam() != other->GetTeam() && true == other->HasFootball())
		//	collideDist = (collideDist * 0.5f) - 0.05f;

		float collideDist = mRadius + other->mRadius;
		if (TEAM_AWAY == GetTeam() && true == other->HasFootball())
			collideDist = (collideDist * 0.4f) - 0.05f;

		Vector3Subtract(&dist, &pos, &other->GetPosition());
		if (Vector3MagnitudeSq(&dist) > (collideDist * collideDist))
			continue;

		
		if ("player" ==  GetRoleName())
		{
			//float d = Vector3DotProduct(&move, &dist);
			//float magDist = Vector3Magnitude(&dist);

			//Vector3 sub;
			//Vector3Scale(&sub, &dist, -d / magDist);
			//Vector3Subtract(&move, &move, &sub);
			mSprite.SetPosition(pos);
		}

		isColliding = true;
		break;
	}

	if (false == isColliding)
		mSprite.SetPosition(pos);
}



class CHumanPlayer : public CFootballPlayer
{
private:
	bool mCanFleaFlick;
public:
	virtual bool IsPlayer(void) const { return true; }

	virtual std::string GetRoleName(void) const { return "player"; }

	virtual void Update(float dt)
	{
		if (false == mFootballField->IsReadyToPlay())
			return;		//The field is not ready to play yet.

		if (true == TheInputSystem->IsKeyPressed(ICEKEY_SPACE))
		{
			Vector3 qbPos = mFootballField->FindPlayerWithRole("QB")->GetPosition();
			if (false == mFootballField->IsInPlay() && true == HasFootball())
			{	//Snap the football.
				mFootballField->HikeFootball();
				mFootball->ThrowTo(qbPos);
				mCanFleaFlick = false;
			}

			if (true == mCanFleaFlick && GetPosition().z < qbPos.z)
			{
				mFootball->ThrowTo(qbPos);
				mCanFleaFlick = false;
			}
		}

		if (GetPosition().z < mFootballField->LineOfScrimmageZ())
			mCanFleaFlick = false;

		bool isRunning = false;
		if (TheInputSystem->IsKeyDown(ICEKEY_UP))
		{
			isRunning = true;
			if (TheInputSystem->IsKeyDown(ICEKEY_LEFT))
				mSprite.SetForward(Vector3(-1.0f, 0.0f, -1.0f));
			else if (TheInputSystem->IsKeyDown(ICEKEY_RIGHT))
				mSprite.SetForward(Vector3(1.0f, 0.0f, -1.0f));
			else
				mSprite.SetForward(Vector3(0.0f, 0.0f, -1.0f));

			Vector3 n = mSprite.GetForward();
			Vector3Normalize(&n, &n);
			mSprite.SetForward(n);
		}
		else if (TheInputSystem->IsKeyDown(ICEKEY_DOWN))
		{
			isRunning = true;
			if (TheInputSystem->IsKeyDown(ICEKEY_LEFT))
				mSprite.SetForward(Vector3(-1.0f, 0.0f, 1.0f));
			else if (TheInputSystem->IsKeyDown(ICEKEY_RIGHT))
				mSprite.SetForward(Vector3(1.0f, 0.0f, 1.0f));
			else
				mSprite.SetForward(Vector3(0.0f, 0.0f, 1.0f));

			Vector3 n = mSprite.GetForward();
			Vector3Normalize(&n, &n);
			mSprite.SetForward(n);
		}
		else if (TheInputSystem->IsKeyDown(ICEKEY_LEFT))
		{
			isRunning = true;
			mSprite.SetForward(Vector3(-1.0f, 0.0f, 0.0f));
		}
		else if (TheInputSystem->IsKeyDown(ICEKEY_RIGHT))
		{
			isRunning = true;
			mSprite.SetForward(Vector3(1.0f, 0.0f, 0.0f));
		}

		if (true == isRunning)
		{
			float runSpeed = BaseRunSpeed;
			float los = mFootballField->LineOfScrimmageZ();
			float firstDown = mFootballField->FirstDownZ();
			float posz = GetPosition().z;

			if (posz > los)
				runSpeed = FastRunSpeed;
			else if (los - posz < 5.0f)
				runSpeed = FastRunSpeed - ((FastRunSpeed - BaseRunSpeed) * ((los - posz) / 5.0f));
			else if (posz > firstDown)
				runSpeed = BaseRunSpeed;
			else if (posz < firstDown && firstDown - posz < 5.0f)
				runSpeed = BaseRunSpeed - ((BaseRunSpeed - SlowRunSpeed) * ((firstDown - posz) / 5.0f));
			else
				runSpeed = SlowRunSpeed;

			RunForward(runSpeed * mRunModifier * dt);
		}
		CFootballPlayer::Update(dt);
	}

	friend class CFootballPlayer;
private:
	CHumanPlayer(void) : mCanFleaFlick(true)
	{
		mSprite.SetTexture(Textures::HomePlayer);
	}
};

const Vector3& CloserOf(const Vector3 &at, const Vector3 &a, const Vector3 &b)
{
	Vector3 distA, distB;
	Vector3Subtract(&distA, &at, &a);
	Vector3Subtract(&distB, &at, &b);
	return (Vector3MagnitudeSq(&distA) < Vector3MagnitudeSq(&distB)) ? a : b;
}

class CComputerPlayer : public CFootballPlayer
{
private:
	bool mDirectRun;
	bool mIndirectAxisX;
	int mTarget;
	bool mIsAsleep;

public:
	
	virtual void Update(float dt)
	{
		if (false == mFootballField->IsInPlay())
			return;

		const Vector3 &playPos = mFootballField->GetPlayerPosition();
		const Vector3 &ballPos = mFootballField->GetFootballPosition();
		const Vector3 &linePos = mFootballField->GetCenterAt(-1);	//Line of Scrimmage.		
		const Vector3 &myPos = GetPosition();

		Vector3 targetPos = playPos;
		if (1 == mTarget)
			targetPos = ballPos;
		if (2 == mTarget)
			targetPos = CloserOf(myPos, playPos, ballPos);

		if (true == mIsAsleep)
		{
			float dim = 2.5;
			if (targetPos.x > myPos.x - dim && targetPos.x < myPos.x + dim
			&&	targetPos.z > myPos.z - dim && targetPos.z < myPos.z + dim)
				mIsAsleep = false;

			return;
		}

		if (true == mDirectRun)
			DirectMoveTowardsTarget(targetPos, dt);
		else
			IndirectMoveTowardsTarget(targetPos, dt);

		CFootballPlayer::Update(dt);
	}

	void DirectMoveTowardsTarget(const Vector3 &targetPos, float dt)
	{
		Vector3 toPlayer;
		Vector3Subtract(&toPlayer, &targetPos, &GetPosition());
		Vector3Normalize(&toPlayer, &toPlayer);
		mSprite.SetForward(toPlayer);
		
		RunForward(BaseRunSpeed * mRunModifier * dt);
	}

	void IndirectMoveTowardsTarget(const Vector3 &targetPos, float dt)
	{
		float dist;
		float aimedModifier = 1.0f;
		Vector3 toPlayer;
		Vector3Subtract(&toPlayer, &targetPos, &GetPosition());		

		if (true == mIndirectAxisX)
		{
			if (abs(toPlayer.z) < 0.25)
				aimedModifier = 2.5f;
			toPlayer.z = 0.0f;
			Vector3NormalizeMag(&toPlayer, &toPlayer, dist);
		}
		else	//Handle the Z-axis
		{
			if (abs(toPlayer.x) < 0.25)
				aimedModifier = 2.5f;
			toPlayer.x = 0.0f;
			Vector3NormalizeMag(&toPlayer, &toPlayer, dist);
		}

		float moveAmt = FastRunSpeed * mRunModifier * aimedModifier * dt;
		if (dist < moveAmt)
		{
			moveAmt = dist;
			mIndirectAxisX = !mIndirectAxisX;
		}

		mSprite.SetForward(toPlayer);
		RunForward(moveAmt);
	}

	friend class CFootballPlayer;
private:
	CComputerPlayer(bool direct) : mDirectRun(direct), mIndirectAxisX((0 == Rand::Rand() % 2)),
		mIsAsleep(true)
	{
		mRunModifier = Rand::RandF(0.8f, 1.01f);
		mSprite.SetForward(Vector3(0.0f, 0.0f, 1.0f));
		mSprite.SetTexture(Textures::AwayPlayer);
		if (true == direct)
			mSprite.GetSprite().SetTextureCoords(0.0f, 0.0f, 0.5f, 0.5f);
		else
			mSprite.GetSprite().SetTextureCoords(0.5f, 0.0f, 0.5f, 0.5f);

		mTarget = Rand::Rand() % 3;
	}
};

//////////////////////////////////////////////////////////////////////////////////////
//																HomeTeamQB
//////////////////////////////////////////////////////////////////////////////////////
class CHomeTeamQB : public CFootballPlayer
{
private:
	Vector3 mThrowPos;
	CSpriteObject mMarker;
	bool mThrowDetermined;

public:

	virtual void Update(float dt)
	{
		if (false == mThrowDetermined)
			DetermineThrow();

		if (false == mFootballField->IsInPlay())
			return;

		if (true == TheInputSystem->IsKeyPressed(ICEKEY_SPACE))
		{
			if (true == HasFootball())
			{
				//DTS_LOG_PASS("QB threw to: %.3f, %.3f\n", mThrowPos.x, mThrowPos.z);
				mFootball->ThrowTo(mThrowPos);
			}
		}

		CFootballPlayer::Update(dt);
	}

	virtual void Draw(void) const
	{
		CFootballPlayer::Draw();

		//if (false == mFootballField->IsInPlay())
		if (false == mFootballField->IsInPlay() || true == HasFootball())
			mMarker.Draw();
	}

	virtual std::string GetRoleName(void) const { return "QB"; }

	friend class CFootballPlayer;
private:
	CHomeTeamQB(void) : mThrowDetermined(false)
	{
		mSprite.SetTexture(Textures::HomeTeam);
		mSprite.GetSprite().SetTextureCoords(0.5f, 0.0f, 0.5f, 0.5f);
		mSprite.mWidth = mSprite.mHeight = 1.5f;

		mMarker.SetTexture(Textures::Football);
		mMarker.GetSprite().SetTextureCoords(0.0f, 0.5f, 0.5f, 0.5f);
		mMarker.mWidth = mMarker.mHeight = 0.5f;
	}

	void DetermineThrow(void)
	{
		float minYard = -1.0f;
		float maxYard = 7.0f;

		int dist = (Rand::Rand() % 10) + 1;
		if (dist <= 5)		{ minYard = 5.0f;		maxYard = 10.0f; }
		else if (dist <= 7)	{ minYard = 2.0f;		maxYard = 5.0f; }
		else				{ minYard = 1.0f;		maxYard = 2.0f; }

		mThrowPos = GetPosition();
		mThrowPos.x += Rand::RandF(-9.0f, 9.0f);
		mThrowPos.z -= Rand::RandF(minYard, maxYard);

		float tossDist;
		Vector3 dir;
		Vector3Subtract(&dir, &mThrowPos, &GetPosition());
		Vector3NormalizeMag(&dir, &dir, tossDist);

		if (tossDist < 2.0f)
			DetermineThrow();

		mMarker.SetPosition(mThrowPos);
		mSprite.SetForward(dir);
		mThrowDetermined = true;

		//DTS_LOG_PASS("QB will throw to: %.3f, %.3f\n", mThrowPos.x, mThrowPos.z);
	}
};


//////////////////////////////////////////////////////////////////////////////////////
//																HomeTeamBlocker
//////////////////////////////////////////////////////////////////////////////////////

static const float PopupDistance = 1.5f;
static const float PopupInfinite = 100.0f;
static const float PopupDropTime = 2.0f;
static const float PopupImageFlip = 0.2f;

class CHomeTeamBlocker : public CFootballPlayer
{
private:
	Vector3 mFalsePos;
	bool mPoppedUp;
	float mTimeLeft;
	float mImageFlip;
	bool mFlipped;

public:
	virtual std::string GetRoleName(void) const { return "HomeBlocker"; }
	virtual bool CanCatch(void) const { return false; }

	virtual void Update(float dt)
	{
		const CFootballPlayer *plyr = mFootballField->FindPlayerWithRole("player");

		if (false == mPoppedUp)
		{	//Check if the player is within a range, then popup!
			Vector3 diff;
			Vector3Subtract(&diff, &plyr->GetPosition(), &mSprite.GetPosition());
			if (Vector3MagnitudeSq(&diff) < PopupDistance * PopupDistance
			||	plyr->GetPosition().z < mSprite.GetPosition().z)
				OnPopUp();
		}

		if (true == mPoppedUp && GetPosition().z > plyr->GetPosition().z + 5.0f)
			OnPopDown();

		if (true == mPoppedUp && mTimeLeft < PopupInfinite)
		{
			mImageFlip -= dt;
			if (mImageFlip < 0.0f)
			{
				mFlipped = !mFlipped;
				mImageFlip = PopupImageFlip;
				if (true == mFlipped)
					mSprite.GetSprite().SetTextureCoords(0.0f, 0.5f, 0.25f, 0.25f);
				else
					mSprite.GetSprite().SetTextureCoords(0.0f, 0.25f, 0.25f, 0.25f);
			}

			if (mTimeLeft >= 0.0f && mFootballField->IsInPlay())
			{
				mTimeLeft -= dt;
				if (mTimeLeft < 0.0f)
					OnPopDown();
			}
		}

		CFootballPlayer::Update(dt);
	}

	const Vector3& GetPosition(void) const
	{	//no collision unless popped up.
		if (false == mPoppedUp)
			return mFalsePos;

		return CFootballPlayer::GetPosition();
	}

	void OnPopUp(void)
	{
		mPoppedUp = true;
		mSprite.GetSprite().SetTextureCoords(0.0f, 0.25f, 0.25f, 0.25f);
	}

	void OnPopDown(void)
	{
		mPoppedUp = false;
		mSprite.GetSprite().SetTextureCoords(0.0f, 0.0f, 0.25f, 0.25f);
	}
	
	friend class CFootballPlayer;
private:

	CHomeTeamBlocker(bool timed) : mPoppedUp(false), mFalsePos(-100.0f, 0.0f, -100.0f),
		mImageFlip(PopupImageFlip), mFlipped(false), mTimeLeft((timed) ? PopupDropTime : PopupInfinite)
	{
		mSprite.SetTexture(Textures::HomeTeam);
		mSprite.GetSprite().SetTextureCoords(0.0f, 0.0f, 0.25f, 0.25f);
	}
};

void CFootballPlayer::CreatePlayer(CFootballPlayer** player, PlayerType type, FootballTeam team)
{
	switch (type)
	{
	case PLYR_ALONE_MAN:		*player = new CHumanPlayer();			break;
	case PLYR_HOME_QB:			*player = new CHomeTeamQB();			break;
	case PLYR_HOME_BLOCK:		*player = new CHomeTeamBlocker(false);	break;
	case PLYR_HOME_TBLOCK:		*player = new CHomeTeamBlocker(true);	break;
	case PLYR_AWAY_DIRECT:		*player = new CComputerPlayer(true);	break;	
	case PLYR_AWAY_INDIRECT:	*player = new CComputerPlayer(false);	break;	
	};

	if (nullptr != *player)
	{
		(*player)->mTeam = team;
	}
}

void CFootballPlayer::DestroyPlayer(CFootballPlayer** player)
{
	delete *player;
	*player = nullptr;
}