/*~******************************************************************************************
*	~Project:	"Alone"																		*
*	~Author:	Tim Beaudet																	*
*	~Modified:	12/16/2011																	*
*	~Purpose:	Common includes, defines and constants.										*
******************************************************************************************~*/

#ifndef _Alone_CFootballPlayer_h_
#define _Alone_CFootballPlayer_h_

static const float SlowRunSpeed = 5.0f / 1.5f;
static const float BaseRunSpeed = 6.0f / 1.5f;
static const float FastRunSpeed = 7.0f / 1.5f;
//static const float SlowRunSpeed = 5.5f;
//static const float BaseRunSpeed = 6.0f;
//static const float FastRunSpeed = 6.5f;

enum PlayerType
{
	PLYR_ALONE_MAN = 0,
	PLYR_HOME_QB = 1,
	PLYR_HOME_BLOCK = 2,
	PLYR_HOME_TBLOCK = 3,
	PLYR_AWAY_DIRECT = 4,
	PLYR_AWAY_INDIRECT = 5,
};

class CFootballField;
class CFootball;

class CFootballPlayer
{
private:
	FootballTeam mTeam;
	float mRadius;

protected:
	CFootball *mFootball;
	CFootballField *mFootballField;
	CSpriteObject mSprite;	

	float mRunModifier;

public:

	virtual void InitWith(void *data) { }
	virtual void Update(float dt) = 0;
	virtual void Draw(void) const;
	virtual bool IsPlayer(void) const { return false; }	
	virtual bool CanCatch(void) const { return true; }
	virtual bool HasFootball(void) const { return (nullptr == mFootball) ? false : true; }
	virtual std::string GetRoleName(void) const { return "unknown"; }

	void RunForward(float amt);
	float GetRadius(void) const { return mRadius; }
	virtual const Vector3& GetForward(void) const { return mSprite.GetForward(); }
	virtual const Vector3& GetPosition(void) const { return mSprite.GetPosition(); }
	virtual void SetPosition(const Vector3 &pos) { mSprite.SetPosition(pos); }
	FootballTeam GetTeam(void) const { return mTeam; }
	virtual void SetFootball(CFootball *ball) { mFootball = ball; }

	static void CreatePlayer(CFootballPlayer** player, PlayerType type, FootballTeam team);
	static void DestroyPlayer(CFootballPlayer** player);

	friend class CFootballField;
protected:
	CFootballPlayer(void);
	virtual ~CFootballPlayer(void);
};

#endif _Alone_CFootballPlayer_h_
