/*~******************************************************************************************
*	~Project:	"Alone"													*
*	~Author:	Tim Beaudet																	*
*	~Modified:	12/16/2011																	*
******************************************************************************************~*/

#include "CStateMachine.h"
#include <assert.h>

CStateMachine theRealStateMachine;
CStateMachine *TheStateMachine = &theRealStateMachine;

CStateMachine::CStateMachine(void) : mCurrState(nullptr)
{

}

CStateMachine::~CStateMachine(void)
{
	ClearStates();
}

void CStateMachine::ClearStates(void)
{
	ChangeToState("");
	while (!mStates.empty())
	{
		delete mStates.back();
		mStates.pop_back();
	}
}

void CStateMachine::AddState(IState *state)
{
	assert(state && nullptr == FindState(state->StateName()) && "Adding invalid, or duplicate state.");
	mStates.push_back(state);
}

bool CStateMachine::ChangeToState(const std::string &strStateName)
{
	IState *nextState = FindState(strStateName);

	if (true == IsStateValid())
		mCurrState->Unload(nextState);

	if (nullptr == nextState || false == nextState->Load(mCurrState))
	{	//Failed to load the state, for unknown reason.
		mCurrState = nullptr;
		return false;
	}

	mCurrState = nextState;
	return false;
}

void CStateMachine::Simulate(void)
{
	if (mCurrState)
		mCurrState->Simulate();
}

void CStateMachine::Update(float dt)
{
	if (true == IsStateValid())
		mCurrState->Update(dt);
}

void CStateMachine::Render(void) const
{
	if (true == IsStateValid())
		mCurrState->Render();
}

void CStateMachine::Draw(void) const
{
	if (true == IsStateValid())
		mCurrState->Draw();
}

bool CStateMachine::IsStateValid(void) const
{
	return (nullptr == mCurrState) ? false : true;
}

bool CStateMachine::DoesStateSetupScene(void) const
{
	if (mCurrState)
		return mCurrState->DoesStateSetupScene();
	return false;
}

IState* CStateMachine::FindState(const std::string &strStateName) const
{
	for (StateContainer::const_iterator itr = mStates.begin(), itrEnd = mStates.end(); itr != itrEnd; ++itr)
		if ((*itr)->StateName() == strStateName)
			return *itr;

	return nullptr;
}