/*~******************************************************************************************
*	~Project:	"Alone"													*
*	~Author:	Tim Beaudet																	*
*	~Modified:	12/16/2011																	*
******************************************************************************************~*/

#include <string>
#include <vector>

#ifndef _Alone_CStateMachine_h_
#define _Alone_CStateMachine_h_

class IState
{
public:
	virtual ~IState(void) = 0 { }

	virtual std::string StateName(void) const = 0 { return ""; }

	/*~**************************************************************************************
	*	~Function:	"Simulate"																*
	*	~Modified:	4/20/2011																*
	*	~Purpose:	Called 0 to N times per frame, at fixed time intervals.					*
	**************************************************************************************~*/
	virtual void Simulate(void) { }

	/*~**************************************************************************************
	*	~Function:	"Update"																*
	*	~Modified:	4/20/2011																*
	*	~Purpose:	Called once per frame, before Render and Draw.							*
	**************************************************************************************~*/
	virtual void Update(float dt) { }

	/*~**************************************************************************************
	*	~Function:	"Render"																*
	*	~Modified:	4/20/2011																*
	*	~Purpose:	Perform the 3D rendering specific to the state.							*
	**************************************************************************************~*/
	virtual void Render(void) const { }

	/*~**************************************************************************************
	*	~Function:	"Draw"																	*
	*	~Modified:	4/20/2011																*
	*	~Purpose:	Perform the 2D drawing specific to the state.							*
	**************************************************************************************~*/
	virtual void Draw(void) const { }

	/*~**************************************************************************************
	*	~Function:	"Load"																	*
	*	~Modified:	4/20/2011																*
	*	~Purpose:	Load the required resources for the state.								*
	*				Returns false, ONLY if the state failed to load.						*
	**************************************************************************************~*/
	virtual bool Load(IState *prevState) { return true; }

	/*~**************************************************************************************
	*	~Function:	"Unload"																*
	*	~Modified:	4/20/2011																*
	*	~Purpose:	Unload, cleanup any resources allocated by the state.					*
	**************************************************************************************~*/
	virtual void Unload(IState *nextState) { }

	virtual bool DoesStateSetupScene(void) const { return false; }
};



class CStateMachine
{
private:
	typedef std::vector<IState*> StateContainer;
	StateContainer mStates;
	IState *mCurrState;

public:
	CStateMachine(void);
	~CStateMachine(void);

	void ClearStates(void);
	void AddState(IState *state);
	bool ChangeToState(const std::string &strStateName);

	void Simulate(void);
	void Update(float dt);
	void Render(void) const;
	void Draw(void) const;

	bool IsStateValid(void) const;
	bool DoesStateSetupScene(void) const;

private:
	CStateMachine(const CStateMachine &obj);
	CStateMachine& operator=(const CStateMachine &obj);

	IState* FindState(const std::string &strStateName) const;
};

extern CStateMachine* TheStateMachine;

#endif _Alone_CStateMachine_h_