/*~******************************************************************************************
*	~Project:	"Alone"																		*
*	~Author:	Tim Beaudet																	*
*	~Modified:	12/16/2011																	*
******************************************************************************************~*/

#include "../Alone.h"
#include "CTutorial.h"

CTutorial::CTutorial(void) : mMaxExitTimer(1.0f), mExitTimer(-1.0f), mTutorialTexture(nullptr)
{

}

CTutorial::~CTutorial(void)
{

}



void CTutorial::Simulate(void)
{

}

void CTutorial::Update(float dt)
{
	if (TheInputSystem->IsKeyPressed(ICEKEY_SPACE) || TheInputSystem->IsKeyPressed(ICEKEY_ESCAPE))
	{
		if (mExitTimer < 0.0f)
			mExitTimer = mMaxExitTimer;
	}

	if (mExitTimer >= 0.0f)
	{
		mExitTimer -= dt;
		if (mExitTimer < 0.0f)
			TheStateMachine->ChangeToState("GamePlay");
	}	
}

void CTutorial::Render(void) const
{

}

void CTutorial::Draw(void) const
{
	float aspect = TheVisualSystem->GetAspectRatio();
	TheVisualSystem->Clear(0xFF39A533);

	float texSize = 0.85f;
	float texAspect = 512.0f / 232.0f;

	CSprite spr;
	spr.SetScreenCoords(0.5f * aspect, 0.5f, texSize, texSize);
	spr.CenterHotSpot();
	spr.SetColor(0xFF00CCFF);

	if (mExitTimer >= 0.0f)
	{
		unsigned long b = (unsigned long)((mExitTimer / mMaxExitTimer) * 256);
		unsigned long a = 0x00FFFFFF | (b << 24);
		spr.SetColor(spr.GetColor() & a);
	}

	TheVisualSystem->Texture_Bind(0, mTutorialTexture);
	spr.Draw();
}

bool CTutorial::Load(IState *prevState)
{
	TheVisualSystem->Texture_CreateFromFile(&mTutorialTexture, ToDataDir() + "Tutorial.png");
	return true;
}

void CTutorial::Unload(IState *nextState)
{
	TheVisualSystem->Texture_Destroy(&mTutorialTexture);		mTutorialTexture = nullptr;
}