
/*~**********************************************************************************************************************
*	~Project / File:	"iceHelpers" - "CSprite.cpp"																		*
*																														*
*	~Author:			Tim Beaudet																						*
*	~Date Created:		10/2/2010																						*
*	~Last Updated:		10/2/2010																						*
*	~Version:			1.0.0																							*
*																														*
*	~Purpose:			A quick sprite helper to make a quad that can be colored, textured or both.  Rotated etc.		*
*																														*
**********************************************************************************************************************~*/

#include "../Alone.h"
#include "CSprite.h"
#include <GameMath.h>

#include <InternalCombustion.h>
using namespace InternalCombustion;
using namespace InternalCombustion::Visual;

unsigned long CSprite::mIndices[6] = { 0, 2, 1, 3, 2, 0 };

CSprite::CSprite(void) : texU(0.0f), texV(0.0f), texW(1.0f), texH(1.0f), scrX(0.0f), scrY(0.0f), scrW(1.0f), scrH(1.0f),
						 hotX(0.0f), hotY(0.0f), color(0xFFFFFFFF), mIsDirty(true)
{

}

CSprite::~CSprite(void)
{

}

void CSprite::Draw(void) const
{
	if (true == mIsDirty)
		Recompute();

	TheVisualSystem->RenderTris(mVertices, mIndices, 4, 2);
}

void CSprite::DrawEx(float orientation) const
{
	GameMath::Matrix4 ident, rot, tran, world;
	GameMath::MatrixCreateIdentity(&ident);
	GameMath::MatrixCreateTranslation(&tran, scrX, scrY, 0.0f);
	GameMath::MatrixCreateRotationZ(&rot, -orientation);

	GameMath::MatrixMultiply(&world, &rot, &tran);
	GameMath::MatrixCreateTranslation(&tran, -scrX, -scrY, 0.0f);
	GameMath::MatrixMultiply(&world, &tran, &world);

	//for (int v = 0; v < 4; ++v)
	//{	//Must convert to screen coordinates...
	//	quad[v].xyz[0] -= spr->hotX;
	//	quad[v].xyz[1] -= spr->hotY;
	//}

	TheVisualSystem->TransformOfWorld(world);
	Draw();
	TheVisualSystem->TransformOfWorld(ident);	//Do we?
}

//float CSprite::ComputeAspectRatio(void)
//{
//	return ui::ComputeAspectRatio();
//}

void CSprite::SetTexturePos(float u, float v)
{
	texU = u; texV = v;
	mIsDirty = true;
}

void CSprite::SetTextureCoords(float u, float v, float w, float h)
{
	SetTexturePos(u, v);
	texW = w; texH = h;
	mIsDirty = true;
}

void CSprite::SetScreenPos(float x, float y)
{
	scrX = x; scrY = y; 
	mIsDirty = true;
}

void CSprite::SetScreenCoords(float x, float y, float w, float h)
{
	SetScreenPos(x, y);
	scrW = w; scrH = h; 	
	mIsDirty = true;
}

void CSprite::SetHotSpot(float x, float y) 
{ 
	hotX = x; hotY = y; 
	mIsDirty = true;
}

void CSprite::CenterHotSpot(void)
{
	SetHotSpot(scrW * 0.5f, scrH * 0.5f);
}

void CSprite::SetColor(unsigned long col) 
{
	color = col; 
	mIsDirty = true;
}

void CSprite::Recompute(void) const
{
	mVertices[0].z = mVertices[1].z = mVertices[2].z = mVertices[3].z = 0.0f;
	mVertices[0].color = mVertices[1].color = mVertices[2].color = mVertices[3].color = color;

	mVertices[0].x = scrX;			mVertices[0].y = scrY;			mVertices[0].u = texU;			mVertices[0].v = texV;
	mVertices[1].x = scrX;			mVertices[1].y = scrY + scrH;	mVertices[1].u = texU;			mVertices[1].v = texV + texH;
	mVertices[2].x = scrX + scrW;	mVertices[2].y = scrY + scrH;	mVertices[2].u = texU + texW;	mVertices[2].v = texV + texH;
	mVertices[3].x = scrX + scrW;	mVertices[3].y = scrY;			mVertices[3].u = texU + texW;	mVertices[3].v = texV;

	for (int v = 0; v < 4; ++v)
	{
		mVertices[v].x -= hotX;
		mVertices[v].y -= hotY;
	}

	mIsDirty = false;
}

#include "../Utilities/Camera2D.h"

CSpriteObject::CSpriteObject(iceTexture *tex) : mTexture(tex), 
	mForward(0.0f, 0.0f, -1.0f), mWidth(1.0f), mHeight(1.0f), mWorldSpace(true)
{

}

CSpriteObject::~CSpriteObject(void)
{

}

void CSpriteObject::Draw(void) const
{
	if (true == mWorldSpace)
	{
		GameMath::Vector3 pos = mPosition;
		Camera::ToScreenSpace(&pos);

		TheVisualSystem->Texture_Bind(0, mTexture);
	//	mSprite.SetScreenPos(pos.x, pos.y);		//orig
		mSprite.SetScreenCoords(pos.x, pos.y, mWidth * Camera::GetZoom(), mHeight* Camera::GetZoom());
		mSprite.CenterHotSpot();

		//float hw = mWidth * Camera::GetZoom() * 0.5f;
		//float hh = mHeight * Camera::GetZoom() * 0.5f;
		//mSprite.SetScreenCoords(pos.x - hw, pos.y - hh, pos.x + hw, pos.y + hh);
	}
	else
	{
		TheVisualSystem->Texture_Bind(0, mTexture);
		mSprite.SetScreenCoords(mPosition.x, mPosition.y, mWidth, mHeight);
		mSprite.CenterHotSpot();

	}

	float o = ForwardVector3ToOrientation(&mForward);
	mSprite.DrawEx(o);
}