/*~**********************************************************************************************************************
*	~Project / File:	"iceHelpers" - "CSprite.h"																		*
*																														*
*	~Author:			Tim Beaudet																						*
*	~Date Created:		10/2/2010																						*
*	~Last Updated:		10/2/2010																						*
*	~Version:			1.0.0																							*
*																														*
*	~Purpose:			A quick sprite helper to make a quad that can be colored, textured or both.  Rotated etc.		*
*																														*
**********************************************************************************************************************~*/

#ifndef _iceHelpers_CSprite_h_
#define _iceHelpers_CSprite_h_

#include "../Alone.h"
#include <GameMath.h>

/*~**********************************
*      Forward Declarations		    *
**********************************~*/

class CSprite
{
private:
	static unsigned long mIndices[6];
	mutable iceTriVertex mVertices[4];
	float texU, texV, texW, texH;
	float scrX, scrY, scrW, scrH;
	float hotX, hotY;
	unsigned long color;
	mutable bool mIsDirty;

public:
	/*~**********************************
	*      Constructors/Destructor      *
	**********************************~*/

	CSprite(void);
	~CSprite(void);

	/*~**********************************
	*       Methods and Functions       *
	**********************************~*/

	void Draw(void) const;
	void DrawEx(float orientation) const;

	/*~**********************************************************************************************************************
	*	~Function:			"MethodName"																					*
	*	~Modified By:		Tim Beaudet																						*
	*	~Last Updated:		3/27/2010																						*
	*																														*
	*	~Purpose:			TEMPLATE_TODO: Write something useful.															*
	*																														*
	*	~In:				Nothing.																						*
	*																														*
	*	~Out:				Nothing.																						*
	**********************************************************************************************************************~*/

	/*~**********************************
	*        Operator Overloads         *
	**********************************~*/

	/*~**********************************
	*      Accessors and Mutators       *
	**********************************~*/

	//static float ComputeAspectRatio(void);

	void SetTexturePos(float u, float v);
	void SetTextureCoords(float u, float v, float w, float h);
	void SetScreenPos(float x, float y);
	void SetScreenCoords(float x, float y, float w, float h);
	void SetHotSpot(float x, float y);
	void CenterHotSpot(void);
	void SetColor(unsigned long col);

	Vector2 GetScreenPos(void) const { Vector2 pos(scrX, scrY); return pos; }
	float GetScreenWidth(void) const { return scrW; }
	float GetScreenHeight(void) const { return scrH; }
	unsigned long GetColor(void) const { return color; }

	/*~**********************************
	*    Friend Classes and Methods     *
	**********************************~*/

	/*~**********************************
	*        Protected Interface        *
	**********************************~*/
protected:

	/*~**********************************
	*         Private Interface         *
	**********************************~*/
private:
	void Recompute(void) const;
};


class CSpriteObject
{
private:


protected:
	GameMath::Vector3 mPosition;	//X,Z
	GameMath::Vector3 mForward;
	mutable CSprite mSprite;
	iceTexture *mTexture;
	bool mWorldSpace;

public:
	float mWidth;
	float mHeight;

public:
	CSpriteObject(iceTexture *tex = nullptr);
	~CSpriteObject(void);

	virtual void Draw(void) const;

	void SetTexture(iceTexture *tex) { mTexture = tex; }	
	void SetForward(const Vector3 &fwd) { mForward = fwd; }
	void SetPosition(const Vector3 &pos) { mPosition = pos; }

	CSprite& GetSprite(void) { return mSprite; }
	const CSprite& GetSprite(void) const { return mSprite; }
	const GameMath::Vector3& GetPosition(void) const { return mPosition; }
	const GameMath::Vector3& GetForward(void) const { return mForward; }

private:
	CSpriteObject(const CSpriteObject &obj);
	CSpriteObject& operator=(const CSpriteObject &obj);	
};

#endif _iceHelpers_CSprite_h_