/*~**********************************************************************************************************************
*	~Project / File:	"TurnBasedRacer" - "CGSSplash.cpp"																*
*																														*
*	~Author:			Tim Beaudet																						*
*	~Date Created:		5/15/2010																						*
*	~Last Updated:		10/2/2010																						*
*	~Version:			1.0.0																							*
*																														*
*	~Purpose:			A quick splash screen of the "BlackBird" logo, which will go into other splashes etc...			*
*																														*
**********************************************************************************************************************~*/

#include "CSpriteFont.h"
#include <GameMath.h>
#include <stdarg.h>	//va_list, va_start, va_end

#include <InternalCombustion.h>
using namespace InternalCombustion;
using namespace InternalCombustion::Visual;

CSpriteFont::CSpriteFont(void)
{
	float letterWidth = 1.0f / 16.0f;
	float letterHeight = 1.0f / 16.0f;

	for (int y = 0; y < 16; y++)
	{
		for (int x = 0; x < 16; x++)
		{
			CSprite &spr = mLetterSprites[x + (y * 16)];
			spr.SetTextureCoords(x * letterWidth, y * letterHeight, letterWidth, letterHeight);
			spr.SetHotSpot(0.0f, 0.0f);
		}
	}
}

CSpriteFont::~CSpriteFont(void)
{

}

void CSpriteFont::Draw(float x, float y, float letterHeight, const char *szText) const
{
	float letterWidth = letterHeight;
	size_t len = strlen(szText);
	for (size_t c = 0; c < len; ++c)
	{
		CSprite &spr = mLetterSprites[szText[c]];
		spr.SetScreenCoords(x + (c * letterWidth * 0.65f), y, letterWidth, letterHeight);
		spr.Draw();
	}
}

void CSpriteFont::DrawFmt(float x, float y, float letterHeight, const char *szTextFmt, ...) const
{
	char buff[1024];
	va_list arg;
	va_start(arg, szTextFmt);
	vsprintf_s(buff, 1024, szTextFmt, arg);
	va_end(arg);

	Draw(x, y, letterHeight, buff);
}

void CSpriteFont::DrawShadowed(float x, float y, float letterHeight, unsigned long shadowColor, const char *szText) const
{
	unsigned long origColor = mLetterSprites[0].GetColor();
	SetColor(shadowColor);
	Draw(x+0.0035f, y+0.0035f, letterHeight, szText);
	SetColor(origColor);
	Draw(x, y, letterHeight, szText);
}

void CSpriteFont::DrawShadowedFmt(float x, float y, float letterHeight, unsigned long shadowColor, const char *szTextFmt, ...) const
{
	char buff[1024];
	va_list arg;
	va_start(arg, szTextFmt);
	vsprintf_s(buff, 1024, szTextFmt, arg);
	va_end(arg);

	DrawShadowed(x, y, letterHeight, shadowColor, buff);
}

float CSpriteFont::ComputeWidth(float letterHeight, const char *szText)
{
	size_t charCount = strlen(szText);
	return charCount * letterHeight * 0.65f;
}

float CSpriteFont::ComputeWidthFmt(float letterHeight, const char *szTextFmt, ...)
{
	char buff[1024];
	va_list arg;
	va_start(arg, szTextFmt);
	vsprintf_s(buff, 1024, szTextFmt, arg);
	va_end(arg);

	return ComputeWidth(letterHeight, buff);
}

void CSpriteFont::SetColor(unsigned long color) const
{
	for (int c = 0; c < 256; ++c)
		mLetterSprites[c].SetColor(color);
}