/*~**********************************************************************************************************************
*	~Project / File:	"iceHelpers" - "VisualPath.h"																	*
*																														*
*	~Author:			Tim Beaudet																						*
*	~Date Created:		11/1/2010																						*
*	~Last Updated:		11/1/2010																						*
*	~Version:			1.0.0																							*
*																														*
*	~Purpose:			Creates an indexed-vertex buffer from a path of vertices.										*
*																														*
**********************************************************************************************************************~*/

#include "../TurnBasedRacer.h"
#include "VisualPath.h"
#include "../RaceTrack/CRaceTrack.h"	//For the WorldToScreenSpace.

CVisualPath::CVisualPath(const std::vector<GameMath::Vector3> &centerLine, float constWidth, float textureScaleU, float textureScaleV, unsigned long color)
	: mCenterLine(centerLine), mConstWidth(constWidth), mTextureScaleU(textureScaleU), mTextureScaleV(textureScaleV), mColor(color), mIsDirty(true)
{
	Recompute();
}

CVisualPath::~CVisualPath(void)
{

}

void CVisualPath::Draw(void) const
{
	if (true == NeedsToRecompute())
		Recompute();

	//TODO: OPTIMIZE:
	//	Instead of making a new buffer, converting from world to screen each _frame_
	//	why not move the camera and keep this stuff in WorldSpace.  As they say,
	//	"hindsight is 20/20", unfortunately the project started with a stubbed leg.
	std::vector<iceTriVertex> verts;
	verts.resize(mVertices.size());
	for (size_t idx = 0; idx < mVertices.size(); ++idx)
	{
		verts[idx] = mVertices[idx];
		CRaceTrack::WorldToScreenSpace(*(Vector3*)&verts[idx]);
	}

	TheVisualSystem->RenderTris(&verts[0], &mIndices[0], mVertices.size(), mIndices.size() / 3);
}

void CVisualPath::Recompute(void) const
{
	if (mCenterLine.size() < 2)
		return;

	mIsDirty = false;
	mVertices.clear();
	mIndices.clear();

	Vector3 temp, right, forward, worldup(0.0f, 1.0f, 0.0f);

	float halfConstWidth = mConstWidth * 0.5f;

	mVertices.resize(2 * mCenterLine.size());
	mIndices.reserve(mVertices.size() * 3);
	for (size_t prevIdx = 0, currIdx = 1, centerIdx = 1, lastIdx = mCenterLine.size(); centerIdx < lastIdx; centerIdx++, prevIdx = currIdx, currIdx += 2)
	{
		Vector3Subtract(&forward, &mCenterLine[centerIdx], &mCenterLine[centerIdx-1]);
		Vector3Normalize(&forward, &forward);
		Vector3CrossProduct(&right, &forward, &worldup);

		Vector3Scale(&temp, &right, halfConstWidth);
		Vector3Add((Vector3*)&mVertices[currIdx].x, &mCenterLine[centerIdx], &temp);
		Vector3Scale(&temp, &right, -halfConstWidth);
		Vector3Add((Vector3*)&mVertices[currIdx+1].x, &mCenterLine[centerIdx], &temp);

		mVertices[currIdx].y = mVertices[currIdx+1].y = 0.2f;

		//TODO: Setup texture coords.

		mVertices[currIdx].color = mColor;
		mVertices[currIdx+1].color = mColor;

		mIndices.push_back(prevIdx);		mIndices.push_back(prevIdx+1);		mIndices.push_back(currIdx);
		mIndices.push_back(prevIdx+1);		mIndices.push_back(currIdx+1);		mIndices.push_back(currIdx);
	}
}