/*~**********************************************************************************************************************
*	~Project / File:	"iceHelpers" - "VisualPath.h"																	*
*																														*
*	~Author:			Tim Beaudet																						*
*	~Date Created:		11/1/2010																						*
*	~Last Updated:		11/1/2010																						*
*	~Version:			1.0.0																							*
*																														*
*	~Purpose:			Creates an indexed-vertex buffer from a path of vertices.										*
*																														*
**********************************************************************************************************************~*/

#ifndef _iceHelpers_VisualPath_h_
#define _iceHelpers_VisualPath_h_

#include <GameMath.h>

/*~**********************************
*      Forward Declarations		    *
**********************************~*/

class CVisualPath
{
private:
	mutable std::vector<iceTriVertex> mVertices;	//The outside world doesn't need to know that this changes
	mutable std::vector<unsigned long> mIndices;	//So it can change / be updated in Draw.
	mutable bool mIsDirty;

	std::vector<GameMath::Vector3> mCenterLine;

	float mConstWidth;
	float mTextureScaleU;
	float mTextureScaleV;
	unsigned long mColor;

public:
	/*~**********************************
	*      Constructors/Destructor      *
	**********************************~*/

	CVisualPath(const std::vector<GameMath::Vector3> &centerLine, float constWidth, float textureScaleU, float textureScaleV, unsigned long color);
	~CVisualPath(void);

	/*~**********************************
	*       Methods and Functions       *
	**********************************~*/

	void Draw(void) const;

	/*~**********************************************************************************************************************
	*	~Function:			"MethodName"																					*
	*	~Modified By:		Tim Beaudet																						*
	*	~Last Updated:		3/27/2010																						*
	*																														*
	*	~Purpose:			TEMPLATE_TODO: Write something useful.															*
	*																														*
	*	~In:				Nothing.																						*
	*																														*
	*	~Out:				Nothing.																						*
	**********************************************************************************************************************~*/

	/*~**********************************
	*        Operator Overloads         *
	**********************************~*/

	/*~**********************************
	*      Accessors and Mutators       *
	**********************************~*/

	/*~**********************************
	*    Friend Classes and Methods     *
	**********************************~*/

	/*~**********************************
	*        Protected Interface        *
	**********************************~*/
protected:

	/*~**********************************
	*         Private Interface         *
	**********************************~*/
private:
	bool NeedsToRecompute(void) const { return (true == mIsDirty || true == mVertices.empty()) ? true : false; }
	void Recompute(void) const;	//Changes the mutable variables: mVertices, mIndices and mIsDirty
};

#endif _iceHelpers_VisualPath_h_