/*~******************************************************************************************
*	~Project:	"Alone"													*
*	~Author:	Tim Beaudet																	*
*	~Modified:	12/16/2011																	*
*	~Purpose:	Base object that will be managed by the world.								*
******************************************************************************************~*/

#include "CScript.h"

#include <assert.h>
#include <fstream>
#include <DebugToolSet.h>

namespace Utilities
{

	CScript* OpenScriptFromFile(const std::string &strFilePath)
	{
		std::string strData, buff;

		std::ifstream ifs;
		ifs.open(strFilePath.c_str(), std::ios::in);
		if (false == ifs.is_open())
			return nullptr;

		while (!ifs.eof())
		{
			ifs >> buff;
			strData += buff;			
			strData += " ";		//add white space allows script to read correctly.
		}

		ifs.close();
		return OpenScriptFromData(strData);
	}

	CScript* OpenScriptFromData(const std::string &strData)
	{
		CScript *newScript = new CScript();
		newScript->mData.str(strData);
		return newScript;
	}

	void CloseScript(CScript *script)
	{
		delete script;
	}

	CScript::CScript(void) : Version(0), AutoWhite('\0')
	{

	}

	CScript::~CScript(void)
	{

	}

	std::string CScript::Read(void)
	{
		std::string data;
		mData >> data;
		return data;
	}

	std::string CScript::ReadString(void)
	{
		std::string str;
		char c;	
		
		//Eat the white space and grab what should be the opening quotation.
		do { c = mData.get(); } while (c == ' ' || c == '\n' || c == '\t');
		assert('\"' == c && "Opening quote not found in CScript::ReadString");

		//Read characters until the ending quotation.  Add all but quote to str.
		while (!EndOfFile() && '\"' != (c = mData.get())) { str += c; }

		return str;
	}

	void CScript::Write(const std::string &data)
	{
		mData << data;
		SCRIPT_AUTOWHITE
	}

	void CScript::WriteString(const std::string &str)
	{
		mData << '\"' << str << '\"';
		SCRIPT_AUTOWHITE
	}

	bool CScript::SaveToFile(const std::string &strFilePath) const
	{
		std::string strData;
		if (false == SaveToData(strData))
			return false;

		std::ofstream ofs;
		ofs.open(strFilePath.c_str(), std::ios::out);
		if (false == ofs.is_open())
			return false;

		ofs << strData;
		ofs.close();
		return true;
	}

	bool CScript::SaveToData(std::string &strData) const
	{
		strData = mData.str();
		return true;
	}

};	//namespace Utilities